# -*- coding: ascii -*-

###########################################################################
# clive, video extraction utility
# Copyright (C) 2007-2008 Toni Gundogdu
#
# clive is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
###########################################################################

## The classes for writing and reading cache

import md5
import time

try:
    import sqlite3
    _sqlite3_found = 1
except ImportError:
    _sqlite3_found = 0

try:
    from clive.newt import snack as newt
except ImportError, err:
    raise SystemExit('error: %s' % err)

from clive.path import ConfigDir
from clive.progress import Progress

__all__ = ['Cache','Browse']

## The class that wraps Cache access
class Cache:

    ## Constructor
    def __init__(self, say):
        self._say = say
        if not _sqlite3_found: return
        self._conn = sqlite3.connect(ConfigDir().cachefile())
        self._conn.row_factory = sqlite3.Row
        self._cur = self._conn.cursor()
        self._upgrade_if_needed()
        self._create_table()

    ## Destructor
    def __del__(self):
        if not _sqlite3_found: return
        self._cur.execute('vacuum')
        self._cur.close()

    ## Write new cache entry
    #
    # Supresses any insertion errors quietly.
    def write(self, v_info):
        hash = md5.new(v_info['url'].strip('&fmt=18')).hexdigest()
        r = self.read(v_info['url'].strip('&fmt=18'))
        if not r:
            try:
                self._cur.execute('insert into cache(' \
                    'cache_hash, ' \
                    'cache_time, ' \
                    'cache_title, ' \
                    'cache_length, ' \
                    'cache_xurl, ' \
                    'cache_vid, ' \
                    'cache_lowq, ' \
                    'cache_url)' \
                    'values(?,?,?,?,?,?,?,?)',
                    (hash, time.time(), v_info['page_title'], 
                    v_info['length_bytes'], v_info['xurl'], v_info['v_id'],
                    v_info['low_quality'], v_info['url'].strip('&fmt=18')))
            except sqlite3.IntegrityError, err:
                pass # Ignore any "url hash not unique" errors
            except sqlite3.OperationalError, err:
                pass # Ignore any "database locked" etc. errors
        else: # Update existing record (e.g. if quality setting was changed)
            if r['cache_lowq'] != v_info['low_quality']:
                self._cur.execute('update cache set ' \
                    'cache_time=?, ' \
                    'cache_title=?, ' \
                    'cache_length=?, ' \
                    'cache_xurl=?, ' \
                    'cache_vid=?, ' \
                    'cache_lowq=?, ' \
                    'cache_url=? where cache_hash=?',
                    (time.time(), v_info['page_title'], v_info['length_bytes'],
                    v_info['xurl'], v_info['v_id'], v_info['low_quality'],
                    v_info['url'].strip('&fmt=18'), hash))
        self._conn.commit()

    ## Read cache entry for URL
    def read(self, url):
        try:
            hash = md5.new(url.strip('&fmt=18')).hexdigest()
            self._cur.execute('select ' \
                'cache_title,' \
                'cache_length,' \
                'cache_xurl,' \
                'cache_vid, ' \
                'cache_lowq ' \
                'from cache where cache_hash=?', (hash,))
        except sqlite3.OperationalError, err:
            if 'no such column' in err.message:
                # Should database upgrade fail for some reason, re-create
                # the table. This will, of course, remove all existing
                # cache data also. See: http://bugs.debian.org/479315
                self._say('cache: recreate cache due to ' +
                    '"no such column" error')
                self._cur.execute('drop table cache')
                self._create_table()
                return None
            else:
                raise err
        return self._cur.fetchone()

    ## Check if extraction URL has expired
    #
    # Some hosts have their extraction URLs expire after awhile.
    def has_xurl_expired(self, url):
        hosts = [('youtube.', 90),('dailymotion.', 2)]
        for (host, expires_min) in hosts: # Only these hosts req. check
            if host in url.lower():
                hash = md5.new(url.strip('&fmt=18')).hexdigest()
                self._cur.execute('select ' \
                    'cache_time from cache where cache_hash=?', (hash,))
                row = self._cur.fetchone()
                if not row: return 0 # Not found -> not expired
                elapsed = time.time() - float(row['cache_time'])
                gmt = time.gmtime(elapsed)
                return (gmt[3]*60 + gmt[4]) >= expires_min
        return 0 # No such host -> not expired

    ## Updates extraction URL for an URL (hash)
    def update_expired_xurl(self, url, xurl):
        hash = md5.new(url.strip('&fmt=18')).hexdigest()
        self._cur.execute('update cache set cache_time=?, cache_xurl=? ' \
            'where cache_hash=?', (time.time(), xurl, hash))
        self._conn.commit()

    ## Select all cache entries, return cursor
    def select_all(self):
        self._cur.execute('select * from cache')
        return self._cur

    def _create_table(self):
        try:
            self._cur.execute('create table cache(' \
                'cache_id integer primary key, ' \
                'cache_hash text not null unique, ' \
                'cache_time float not null, ' \
                'cache_title text not null, ' \
                'cache_length long not null, ' \
                'cache_xurl text not null, ' \
                'cache_vid text not null,' \
                'cache_lowq integer not null,' \
                'cache_url text not null)')
        except sqlite3.OperationalError, err:
            # Ignore only "table exists" errors only
            if 'table exists already' in err.message:
                pass

    ## Destructor
    # Upgrades table if it appears to be outdated (e.g. missing columns)
    def _upgrade_if_needed(self):
        try:
            self._cur.execute('select * from cache limit 1')
            row = self._cur.fetchone()
            if not row: return
            if len(row) < 8:
                # Must be <= 0.4.10 database
                self._alter_table("'cache_lowq' integer not null default -1")
            if len(row) < 9:
                # Must be > 0.4.10 <= 0.4.16 database
                self._alter_table("'cache_url' text not null default ''")
        except sqlite3.OperationalError, err:
            # Ignore "no such table" errors only
            if err.message == 'no such table' != -1:
                pass

    # Alter table by adding a new column
    def _alter_table(self, s):
        self._cur.execute('alter table cache add column %s' % s)
        self._conn.commit()

## The cache browsing class
class Browse:

    ## Constructor
    def __init__(self, opts, say):
        self._opts = opts
        self._say = say

    ## Browse URLs for re-extraction
    def browse(self):
        self._cache = Cache(self._say)
        cur = self._cache.select_all()

        sel = []
        try:
            scr = newt.SnackScreen()
            sel = self._show_cache(scr, cur)
        finally:
            scr.finish()
        return sel            

    def _show_cache(self, scr, cur):
        ct = newt.CheckboxTree(10, scroll=1)
        while 1:
            row = cur.fetchone()
            if not row: break
            t = row['cache_title'][:52]
            if len(t) == 52: t += '..'
            t += (' (%s)' % Progress(None,None) \
                ._byteshuman(row['cache_length']))
            if len(row['cache_url']): # Ignore any < 0.4.16 entries
                ct.append(t.encode('utf-8'), row['cache_url'])
        g = newt.GridForm(scr, 'Cache: Select to re-extract', 1,2)
        g.add(ct, col=0, row=0, padding=(0,0,0,1))
        b = newt.ButtonBar(scr, [('Cancel',0), ('OK',1)], compact=0)
        g.add(b, col=0, row=01, padding=(0,0,0,0))
        if not b.buttonPressed(g.runOnce()):
            raise SystemExit('Cancelled.')
        return ct.getSelection()
